# AUTOGENERATED! DO NOT EDIT! File to edit: ../../../nbs/torch.dataset.utils.ipynb.

# %% auto 0
__all__ = ['ListDataset']

# %% ../../../nbs/torch.dataset.utils.ipynb 3
from torch.utils.data import Dataset

# %% ../../../nbs/torch.dataset.utils.ipynb 4
class ListDataset(Dataset):
     def __init__(self, original_list):
        self.original_list = original_list
     
     def __len__(self):
        return len(self.original_list)

     def __getitem__(self, i):
        return self.original_list[i]
